//
//  RootViewController.m
//  ThinkNear
//
//  Created by Adam Ernst on 3/7/11.
//  Copyright 2011 cosmicsoft. All rights reserved.
//

#import "TNOfferListViewController.h"
#import "TNOfferListRequestOperation.h"
#import "TNOffer.h"
#import "TNOfferViewController.h"
#import "TNClaim.h"
#import "TNBusiness.h"


@implementation TNOfferListViewController

- (void)commonInit {
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appComingToForeground:) name:UIApplicationDidBecomeActiveNotification object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appComingToForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
}

- (id)initWithCoder:(NSCoder *)aDecoder {
	if ((self = [super initWithCoder:aDecoder])) {
		[self commonInit];
	}
	return self;
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
		[self commonInit];
	}
	return self;
}

- (void)presentError:(NSError *)error {
    [[[[UIAlertView alloc] initWithTitle:[error localizedDescription]
                                 message:nil
                                delegate:nil 
                       cancelButtonTitle:NSLocalizedString(@"Dismiss", @"")
                       otherButtonTitles:nil] autorelease] show];
    
    experiencedError = YES;
}

#pragma mark UIViewController

@synthesize tableView;
@synthesize noOffersCell;
@synthesize refreshItem;

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationDidBecomeActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationWillEnterForegroundNotification object:nil];
    
    [locationManager release];
    [offerListRequestOperation release];
    [offers release];
    [tableView release];
    [noOffersCell release];
    [refreshItem release];
    [statusLabel release];
    [super dealloc];
}

- (void)awakeFromNib {
    locationManager = [[CLLocationManager alloc] init];
    [locationManager setDesiredAccuracy:kCLLocationAccuracyHundredMeters];
    [locationManager setDistanceFilter:300]; // meters
    [locationManager setDelegate:self];
    [locationManager startUpdatingLocation];
    
    statusLabel = [[UILabel alloc] initWithFrame:CGRectMake(30, 0, 160, 20)];
    [statusLabel setFont:[UIFont boldSystemFontOfSize:15.0]];
    [statusLabel setTextColor:[UIColor whiteColor]];
    [statusLabel setText:NSLocalizedString(@"Locating...", @"")];
    [statusLabel setBackgroundColor:[UIColor clearColor]];
    [statusLabel setOpaque:NO];
    
    statusActivityIndicator = [[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0, 0, 20, 20)];
    [statusActivityIndicator setActivityIndicatorViewStyle:UIActivityIndicatorViewStyleWhite];
    [statusActivityIndicator setHidesWhenStopped:YES];
    [statusActivityIndicator startAnimating];
    
    UIView *statusView = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, 190, 20)] autorelease];
    [statusView addSubview:statusLabel];
    [statusView addSubview:statusActivityIndicator];
    
    UIBarButtonItem *statusWrapper = [[[UIBarButtonItem alloc] initWithCustomView:statusView] autorelease];
    
    [self setToolbarItems:[NSArray arrayWithObjects:statusWrapper, 
                           [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil] autorelease],
                           [self refreshItem], nil]];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [[self refreshItem] setEnabled:(offers || experiencedError) && !offerListRequestOperation];
}

- (void)viewDidUnload {
    [self setTableView:nil];
    [self setNoOffersCell:nil];
    [self setRefreshItem:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated {
    [[self tableView] deselectRowAtIndexPath:[[self tableView] indexPathForSelectedRow] animated:YES];
    [super viewWillAppear:animated];
}

- (void)startListOperation {
    if (offerListRequestOperation) {
        [offerListRequestOperation cancel];
        [offerListRequestOperation release], offerListRequestOperation = nil;
    }
    
    [[self refreshItem] setEnabled:NO];
    
#if 1
    CLLocation *location = [[[CLLocation alloc] initWithLatitude:40.743266 longitude:-73.996889] autorelease];
#else
    CLLocation *location = [locationManager location];
#endif
    
    [statusActivityIndicator startAnimating];
    [statusLabel setText:NSLocalizedString(@"Finding offers...", @"")];
    
    TNOfferListRequestOperation *op = [[[TNOfferListRequestOperation alloc] initWithLocation:location] autorelease];
    [op setCompletionBlock:^{
        [self performSelectorOnMainThread:@selector(offerListRequestOperationFinished:) withObject:op waitUntilDone:NO];
		[op setCompletionBlock:nil];
    }];
    offerListRequestOperation = [op retain];
    [op start];
}

#pragma mark Actions

- (IBAction)refresh {
    [self startListOperation];
}

- (void)updatedOffers:(NSArray *)newOffers {
    [offers autorelease], offers = [newOffers retain];
    [[self tableView] reloadData];
}

- (void)errorUpdatingOffers:(NSError *)error {
    [self presentError:error];
    [statusLabel setText:NSLocalizedString(@"", @"")];
    [statusActivityIndicator stopAnimating];
}

- (void)offerListRequestOperationFinished:(TNOfferListRequestOperation *)op {
    assert([NSThread isMainThread]);
    if (offerListRequestOperation != op) return;
    
    if ([op offers]) {
        [self updatedOffers:[op offers]];
    } else if ([op error]) {
        [self errorUpdatingOffers:[op error]];
    }
    
    [statusLabel setText:@""];
    [statusActivityIndicator stopAnimating];
    [[self refreshItem] setEnabled:YES];
    [offerListRequestOperation release], offerListRequestOperation = nil;
}

#pragma mark UITableViewDelegate/UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return MAX([offers count], 1);
}

- (UITableViewCell *)tableView:(UITableView *)table cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"OfferCell";
    
    if ([offers count] == 0) return [self noOffersCell];
    
    UITableViewCell *cell = [table dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
    [cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
    
    TNOffer *offer = [offers objectAtIndex:[indexPath row]];
    
#if 1
    CLLocation *location = [[[CLLocation alloc] initWithLatitude:40.743266 longitude:-73.996889] autorelease];
#else
    CLLocation *location = [locationManager location];
#endif
    
    CLLocationDistance distance = [location distanceFromLocation:[offer location]] / 1609.344;
    
    [[cell textLabel] setText:[offer title]];
    [[cell detailTextLabel] setText:[NSString stringWithFormat:@"%.1f mi (%@)", distance, [[offer business] address1]]];

    return cell;
}

- (void)tableView:(UITableView *)table didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    TNOffer *offer = [offers objectAtIndex:[indexPath row]];
    [[self navigationController] pushViewController:[[[TNOfferViewController alloc] initWithOffer:offer location:[locationManager location]] autorelease] animated:YES];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if ([offers count] == 0) {
        return 120;
    }
    
    return 56;
}

#pragma mark CLLocationManagerDelegate

- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
    [self startListOperation];
    
    if ([[newLocation timestamp] compare:[NSDate dateWithTimeIntervalSinceNow:-60*2]] == NSOrderedDescending
        && [newLocation horizontalAccuracy] < 200) {
        [manager stopUpdatingLocation];
    }
}

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    [self presentError:error];
}

#pragma mark Refreshing

- (void)appComingToForeground:(NSNotification *)notification {
	[offers release], offers = nil;
	[[self tableView] reloadData];
	[self refresh];
}

@end
