//
//  OfferViewController.m
//  ThinkNear
//
//  Created by Adam Ernst on 3/11/11.
//  Copyright 2011 cosmicsoft. All rights reserved.
//

#import "TNOfferViewController.h"
#import "TNOffer.h"
#import "MBProgressHUD.h"
#import "TNClaimRequestOperation.h"
#import "TNClaim.h"
#import "TNBusiness.h"
#import "TNMapViewController.h"
#import "TNButton.h"
#import "TNSaveOfferViewController.h"
#import "TNOfferRequestOperation.h"


@implementation TNOfferViewController

@synthesize titleLabel;
@synthesize descriptionLabel;
@synthesize codeLabel;
@synthesize offerView;
@synthesize offerBackground;
@synthesize codeSection;
@synthesize discountLabel;
@synthesize expirationLabel;
@synthesize timeRemainingLabel;
@synthesize mapCell;
@synthesize phoneCell;
@synthesize addressLabel;
@synthesize phoneLabel;

- (void)updateButtons {
    TNButton *save = [TNButton buttonWithColor:TNButtonColorBlack];
    [save setTitle:NSLocalizedString(@"Save for Later", @"") forState:UIControlStateNormal];
    [save sizeToFit];
    [save addTarget:self action:@selector(save:) forControlEvents:UIControlEventTouchUpInside];
    UIBarButtonItem *saveBarItem = [[[UIBarButtonItem alloc] initWithCustomView:save] autorelease];
    
    [claimButton release], claimButton = nil;

    UIBarButtonItem *redeemBarItem;
    if (!claim) {
        claimButton = [[TNButton buttonWithColor:TNButtonColorRed] retain];
        [claimButton setTitle:NSLocalizedString(@"Redeem to Use Now", @"") forState:UIControlStateNormal];
        [claimButton sizeToFit];
        [claimButton addTarget:self action:@selector(claim:) forControlEvents:UIControlEventTouchUpInside];
        redeemBarItem = [[[UIBarButtonItem alloc] initWithCustomView:claimButton] autorelease];
    } else {
        redeemBarItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil] autorelease];
    }

    [self setToolbarItems:[NSArray arrayWithObjects:redeemBarItem, saveBarItem, nil]];                               
}

- (id)initWithOffer:(TNOffer *)off location:(CLLocation *)loc {
    if ((self = [super initWithNibName:@"Offer" bundle:nil])) {
        offer = [off retain];
        location = [loc retain];
        
        [[self navigationItem] setTitle:NSLocalizedString(@"Offer", @"")];
		
		// Re-requesting the offer is done for server-side analytics only.
		// It is not actually used.
		offerRequestOperation = [[TNOfferRequestOperation alloc] initWithOfferId:[offer identifier]];
		[offerRequestOperation start];
    }
    return self;
}

- (id)initWithClaim:(TNClaim *)cl {
    if ((self = [self initWithOffer:[cl offer] location:nil])) {
        claim = [cl retain];
    }
    return self;
}

- (void)dealloc {
    [claim release];
    [offer release];
    [location release];
    
    [titleLabel release];
    [descriptionLabel release];
    [claimButton release];
    
    [hud release];
    [claimOperation release];
	[offerRequestOperation release];
    
    [codeLabel release];
    [offerBackground release];
    [discountLabel release];
    [codeSection release];
    [offerView release];
    [expirationLabel release];
    [timeRemainingLabel release];
    [mapCell release];
    [phoneCell release];
    [addressLabel release];
    [phoneLabel release];
    [super dealloc];
}

static NSDateFormatter *expirationFormatter;
+ (NSDateFormatter *)expirationFormatter {
    if (expirationFormatter == nil) {
        expirationFormatter = [[NSDateFormatter alloc] init];
        [expirationFormatter setDateFormat:@"MMM d h:mma"];
    }
    return expirationFormatter;
}

- (void)invalidateExpirationTimer {
    if (expirationTimer) {
        [expirationTimer invalidate], [expirationTimer release], expirationTimer = nil;
    }
}

- (void)activateExpirationTimer {
    [self invalidateExpirationTimer]; // In case one is running
    
    if (claim) {
        expirationTimer = [[NSTimer scheduledTimerWithTimeInterval:1.0 
                                                            target:self
                                                          selector:@selector(updateExpirationTime) 
                                                          userInfo:nil
                                                           repeats:YES] retain];
    }
}

- (void)updateExpirationTime {
    NSString *expirationFormat = @"";
    if ([[claim expirationDate] compare:[NSDate date]] == NSOrderedDescending) {
        expirationFormat = NSLocalizedString(@"Expires %@", @"");
        [[self timeRemainingLabel] setTextColor:[UIColor blackColor]];
        
        NSTimeInterval timeRemaining = [[claim expirationDate] timeIntervalSinceDate:[NSDate date]];
        int hours = floor(timeRemaining / (60 * 60));
        timeRemaining -= (hours * 60 * 60);
        int minutes = floor(timeRemaining / 60);
        timeRemaining -= (minutes * 60);
        int seconds = floor(timeRemaining);
        
        NSString *remaining;
        if (hours) {
            remaining = [NSString stringWithFormat:@"%d:%02d:%02d", hours, minutes, seconds];
        } else {
            remaining = [NSString stringWithFormat:@"%d:%02d", minutes, seconds];
        }
        
        [[self timeRemainingLabel] setText:remaining];
    } else {
        expirationFormat = NSLocalizedString(@"Expired %@", @"");
        [[self timeRemainingLabel] setText:@"EXPIRED"];
        [[self timeRemainingLabel] setTextColor:[UIColor redColor]];
    }
    [[self expirationLabel] setText:[NSString stringWithFormat:expirationFormat,
                                     [[TNOfferViewController expirationFormatter] stringFromDate:[claim expirationDate]]]];
}

- (void)reloadFields {
    [self updateButtons];
    
    [[self titleLabel] setText:[offer title]];
    [[self descriptionLabel] setText:[offer description]];
    if ([offer discount]) {
        [[self discountLabel] setText:[NSString stringWithFormat:@"%@", [offer discount]]];
    } else {
        [[self discountLabel] setText:@"—"];
    }
    
    TNBusiness *biz = [offer business];
    [[self addressLabel] setText:[NSString stringWithFormat:NSLocalizedString(@"%@\n%@, %@ %@", @"address"),
                                  [biz address1], [biz city], [biz state], [biz zip]]];
    [[self phoneLabel] setText:[biz phone]];
        
    // Resize labels
    CGRect f;
    CGRect prevFrame;
    CGSize s;
    
    s = [[offer description] sizeWithFont:[[self descriptionLabel] font] constrainedToSize:CGSizeMake([[self descriptionLabel] bounds].size.width, 100)
                            lineBreakMode:UILineBreakModeWordWrap];
    f = [[self descriptionLabel] frame];
    f.size.height = s.height;
    [[self descriptionLabel] setFrame:f];
    prevFrame = f;
    
    [[self codeSection] setHidden:(claim == nil)];
    if (claim) {
        [[self codeLabel] setText:[claim code]];
        f = [[self codeSection] frame];
        f.origin.y = prevFrame.origin.y + prevFrame.size.height + 10;
        [[self codeSection] setFrame:f];
        prevFrame = [[self codeSection] frame];
        
        [self updateExpirationTime];
    }
    
    // Offer background
    UIImage *offerBackgroundImage = [UIImage imageNamed:@"offer_background.png"];
    [[self offerBackground] setImage:[offerBackgroundImage stretchableImageWithLeftCapWidth:0 topCapHeight:100]];
    f = [[self offerBackground] frame];
    f.size.height = prevFrame.size.height + prevFrame.origin.y - f.origin.y + 36;
    [[self offerBackground] setFrame:f];
    prevFrame = f;
        
    [[self offerView] setFrame:CGRectMake(0, 0, [[self offerView] frame].size.width, prevFrame.size.height + (prevFrame.origin.y * 2))];
    [[self tableView] setTableHeaderView:[self offerView]];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [[self view] setBackgroundColor:[UIColor colorWithPatternImage:[UIImage imageNamed:@"background.png"]]];
    
    [self reloadFields];
}

- (void)viewDidUnload {
    [self setTitleLabel:nil];
    [self setDescriptionLabel:nil];
    [self setCodeLabel:nil];
    [self setOfferBackground:nil];
    [self setDiscountLabel:nil];
    [self setCodeSection:nil];
    [self setOfferView:nil];
    [self setExpirationLabel:nil];
    [self setTimeRemainingLabel:nil];
    [self setMapCell:nil];
    [self setPhoneCell:nil];
    [self setAddressLabel:nil];
    [self setPhoneLabel:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated {
    [self activateExpirationTimer];
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    [self invalidateExpirationTimer];
    [super viewWillDisappear:animated];
}

#pragma mark Actions

- (IBAction)claim:(id)sender {
    hud = [[MBProgressHUD alloc] initWithView:[self view]];
    [[self view] addSubview:hud];
    [hud setLabelText:NSLocalizedString(@"Claiming…", @"")];
    [hud show:YES];
    
    [claimButton setEnabled:NO];
    
    claimOperation = [[TNClaimRequestOperation alloc] initWithOffer:offer location:location];
    [claimOperation setCompletionBlock:^{
        [self performSelectorOnMainThread:@selector(claimOperationCompleted) withObject:nil waitUntilDone:NO];
    }];
    [claimOperation start];
}

- (void)hideHud {
    [hud hide:YES];
}

- (void)claimOperationCompleted {
    if ([claimOperation claim]) {
        [hud setCustomView:[[[UIImageView alloc] initWithImage:[UIImage imageNamed:@"claimed.png"]] autorelease]];
        [hud setMode:MBProgressHUDModeCustomView];
        [hud setLabelText:@"Claimed!"];
        [self performSelector:@selector(hideHud) withObject:nil afterDelay:1.0];
        
        claim = [[claimOperation claim] retain];
        claim.expirationDate = [NSDate dateWithTimeIntervalSinceNow:60 * 60];
        [self reloadFields];
        [self activateExpirationTimer];
    } else {
        [hud hide:NO];
        
        [[[[UIAlertView alloc] initWithTitle:[[claimOperation error] localizedDescription] 
                                     message:nil 
                                    delegate:nil 
                           cancelButtonTitle:NSLocalizedString(@"OK", @"") 
                           otherButtonTitles:nil] autorelease] show];
    }
    [claimButton setEnabled:YES];
}

- (void)save:(id)sender {
    [[self navigationController] pushViewController:[[[TNSaveOfferViewController alloc] initWithOffer:offer] autorelease]
                                           animated:YES];
}

#pragma mark UITableViewDataSource/UITableViewDelegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 2;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    switch ([indexPath row]) {
        case 0:
            return [self mapCell];
        case 1:
            return [self phoneCell];
    }
    return nil;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if ([indexPath row] == 0) return 86.0; // extra height for top border
    return 64.0;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    switch ([indexPath row]) {
        case 0:
            [[self navigationController] pushViewController:[[[TNMapViewController alloc] initWithOffer:offer] autorelease] animated:YES];
            break;
        case 1:
            [[[[UIAlertView alloc] initWithTitle:[[offer business] phone] 
                                         message:nil
                                        delegate:self 
                               cancelButtonTitle:NSLocalizedString(@"Cancel", @"") 
                               otherButtonTitles:NSLocalizedString(@"Call", @""), nil] autorelease] show];
            [[self tableView] deselectRowAtIndexPath:[[self tableView] indexPathForSelectedRow] animated:YES];
            break;
    }
}

#pragma mark UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (buttonIndex != [alertView cancelButtonIndex]) {
        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:[NSString stringWithFormat:@"tel:%@", [[offer business] phone]]]];
    }
}

@end

