//
//  TNOffer.m
//  ThinkNear
//
//  Created by Adam Ernst on 3/7/11.
//  Copyright 2011 cosmicsoft. All rights reserved.
//

#import "TNOffer.h"
#import "TNBusiness.h"
#import "TNHelper.h"


@implementation TNOffer

@synthesize identifier, title, description, discount;
@synthesize business, location;
@synthesize category, draws;
@synthesize published, expiration, rescindedAt, validFrom, validTo;

- (id)initWithJSONDictionary:(NSDictionary *)dict {
    if ((self = [super init])) {
        NSDictionary *off = [dict objectForKey:@"offer"];
        
        identifier = [[off objectForKey:@"id"] retain];
        title = [[off objectForKey:@"title"] retain];
        description = [[off objectForKey:@"description"] retain];
        discount = [[off objectForKey:@"discount"] retain];
        
        business = [[TNBusiness alloc] initWithJSONDictionary:[off objectForKey:@"business"]];
        NSDictionary *loc = [off objectForKey:@"location"];
        location = [[CLLocation alloc] initWithLatitude:[[[TNHelper APINumberFormatter] numberFromString:[loc objectForKey:@"latitude"]] doubleValue]
                                              longitude:[[[TNHelper APINumberFormatter] numberFromString:[loc objectForKey:@"longitude"]] doubleValue]];
        
        category = [[off objectForKey:@"category"] retain];
        draws = [[off objectForKey:@"draws"] retain];
        
        NSDateFormatter *f = [TNHelper APIDateFormatter];
        NSDate *(^parseDate)(NSString *) = ^(NSString *key) {
            id s = [off objectForKey:key];
            if (s == [NSNull null]) return (NSDate *)nil;
            return [f dateFromString:s];
        };
        
        published = [parseDate(@"published") retain];
        expiration = [parseDate(@"expiration") retain];
        rescindedAt = [parseDate(@"rescinded_at") retain];
        validFrom = [parseDate(@"valid_from") retain];
        validTo = [parseDate(@"valid_to") retain];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
    if ((self = [super init])) {
        identifier = [[aDecoder decodeObjectForKey:@"identifier"] retain];
        title = [[aDecoder decodeObjectForKey:@"title"] retain];
        description = [[aDecoder decodeObjectForKey:@"description"] retain];
        discount = [[aDecoder decodeObjectForKey:@"discount"] retain];
        
        business = [[aDecoder decodeObjectForKey:@"business"] retain];
        location = [[aDecoder decodeObjectForKey:@"location"] retain];
        
        category = [[aDecoder decodeObjectForKey:@"category"] retain];
        draws = [[aDecoder decodeObjectForKey:@"draws"] retain];
        
        published = [[aDecoder decodeObjectForKey:@"published"] retain];
        expiration = [[aDecoder decodeObjectForKey:@"expiration"] retain];
        rescindedAt = [[aDecoder decodeObjectForKey:@"rescindedAt"] retain];
        validFrom = [[aDecoder decodeObjectForKey:@"validFrom"] retain];
        validTo = [[aDecoder decodeObjectForKey:@"validTo"] retain];
    }
    return self;
}

- (void)dealloc {
    [identifier release];
    [title release];
    [description release];
    [discount release];
    
    [business release];
    [location release];
    
    [category release];
    [draws release];
    
    [published release];
    [expiration release];
    [rescindedAt release];
    [validFrom release];
    [validTo release];
    
    [super dealloc];
}

- (void)encodeWithCoder:(NSCoder *)aCoder {
    [aCoder encodeObject:identifier forKey:@"identifier"];
    [aCoder encodeObject:title forKey:@"title"];
    [aCoder encodeObject:description forKey:@"description"];
    [aCoder encodeObject:discount forKey:@"discount"];
    
    [aCoder encodeObject:business forKey:@"business"];
    [aCoder encodeObject:location forKey:@"location"];
    
    [aCoder encodeObject:category forKey:@"category"];
    [aCoder encodeObject:draws forKey:@"draws"];
    
    [aCoder encodeObject:published forKey:@"published"];
    [aCoder encodeObject:expiration forKey:@"expiration"];
    [aCoder encodeObject:rescindedAt forKey:@"rescindedAt"];
    [aCoder encodeObject:validFrom forKey:@"validFrom"];
    [aCoder encodeObject:validTo forKey:@"validTo"];
}

@end
