//
//  TNOfferRequestOperation.m
//  ThinkNear
//
//  Created by Adam Ernst on 3/7/11.
//  Copyright 2011 cosmicsoft. All rights reserved.
//

#import "TNOfferListRequestOperation.h"
#import "TNHelper.h"
#import "TNOffer.h"


@implementation TNOfferListRequestOperation

- (id)initWithLocation:(CLLocation *)loc {    
    if ((self = [super init])) {
        assert(loc != nil);
        location = [loc retain];
    }
    return self;
}

- (void)dealloc {
    [location release];
    [date release];
    [category release];
    [draw release];
    [minimumDiscount release];
    [offers release];
    [super dealloc];
}

@synthesize date, category, draw, minimumDiscount;

- (void)setDate:(NSDate *)dt {
    assert([self isExecuting] == NO && [self isFinished] == NO);
    [date autorelease], date = [dt retain];
}

- (void)setCategory:(NSString *)cat {
    assert([self isExecuting] == NO && [self isFinished] == NO);
    [category autorelease], category = [cat copy];
}

- (void)setDraw:(NSString *)dr {
    assert([self isExecuting] == NO && [self isFinished] == NO);
    [draw autorelease], draw = [dr copy];
}

- (void)setMinimumDiscount:(NSDecimalNumber *)minDisc {
    assert([self isExecuting] == NO && [self isFinished] == NO);
    [minimumDiscount autorelease], minimumDiscount = [minDisc retain];
}

- (NSArray *)offers {
    return offers;
}

#pragma mark Abstract Methods

- (OAMutableURLRequest *)request {
    CLLocationCoordinate2D coord = [location coordinate];
    NSNumberFormatter *f = [TNHelper APINumberFormatter];
    
    NSMutableDictionary *params = [NSMutableDictionary dictionary];
    if (date) {
        [params setObject:[[TNHelper APIDateFormatter] stringFromDate:date] forKey:@"time"];
    }
    if (category) {
        [params setObject:category forKey:@"category"];
    }
    if (draw) {
        [params setObject:draw forKey:@"draw"];
    }
    if (minimumDiscount) {
        [params setObject:[f stringFromNumber:minimumDiscount] forKey:@"min_discount"];
    }
    
    NSString *path = [NSString stringWithFormat:@"/offers/%@,%@.json?%@", 
                      [f stringFromNumber:[NSNumber numberWithDouble:coord.latitude]],
                      [f stringFromNumber:[NSNumber numberWithDouble:coord.longitude]],
                      [TNHelper URLQueryStringFromDictionary:params]];
    
    return [self JSONRequestWithRelativeURL:path];
}

- (void)handleResponse:(NSHTTPURLResponse *)response data:(id)jsonData {
    NSMutableArray *newOffers = [NSMutableArray array];
    for (id off in jsonData) {
        if (off != [NSNull null]) {
            [newOffers addObject:[[[TNOffer alloc] initWithJSONDictionary:off] autorelease]];
        }
    }
    [self willChangeValueForKey:@"offers"];
    offers = [newOffers retain];
    [self didChangeValueForKey:@"offers"];
    [self finishWithError:nil];
}

@end
